-------------------------------------------------------
-- Design Name : mDAB_ctrl_tb_01
-- File Name   : mDAB_ctrl_tb_01.vhd
-- Function    : mDAB control test bench
-- Coder       : K.-H. Sulanke, DESY
-- Date        : 2020-12-18
-------------------------------------------------------
library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
 
entity mDAB_ctrl_tb_01 is
  --generic( REG_WIDTH : integer := 32);
end mDAB_ctrl_tb_01;

architecture behavior of mDAB_ctrl_tb_01 is
 
  component mDAB_ctrl_top is 
    port( 
		MCU_CAL_MISO       : out std_logic;  --
		MCU_CAL_MOSI       : in  std_logic;  --
		MCU_CAL_SS         : in  std_logic;  -- low active
		MCU_AFE_SEL        : in  std_logic;  --
		MCU_CAL_SCLK       : in  std_logic;  --
		MCU_CAL_A          : in  std_logic_vector(3 downto 0); -- MCU_CAL_A4 not used
		ADC_CSn            : out std_logic;  --
		ADC_CONVSTnR       : out std_logic;  --
		ADC_SDIR           : out std_logic;  --
		ADC_SCLKR          : out std_logic;  --
		ADC_SDO		       : in  std_logic;  --
		INTLK_2            : in  std_logic;  -- in (!)
		DAC_SCLK           : out std_logic;  -- Flasher Bias voltage control
		DAC_SYNCn          : out std_logic;  --
		DAC_DIN            : out std_logic;  --
		FLASH1_POW_EN      : out std_logic;  --
		FLASH1_SELECT_CLR  : out std_logic;  -- low active
		FLASH1_SELECT_DIN  : out std_logic;  --
		FLASH1_SELECT_CLK  : out std_logic;
		FLASH2_POW_EN      : out std_logic;  --
		FLASH2_SELECT_CLK  : out std_logic;  --
		FLASH2_SELECT_DIN  : out std_logic;  --
		FLASH2_SELECT_CLR  : out std_logic;  -- low active
		ILUM_POW_EN        : out std_logic;  --
		CAM_MISO           : in  std_logic;  --
		CAM_MOSI           : out std_logic;  --
		CAM_SCLK           : out std_logic;  --
		CAM1_POW_EN        : out std_logic;  --
		CAM1_ENA           : out std_logic;  -- high active, = camera_1 power enable
		CAM1_SS            : out std_logic;  -- low active 
		LUM1_ENA           : out std_logic;  --
		CAM2_POW_EN        : out std_logic;  --
		CAM2_ENA           : out std_logic;  --
		CAM2_SS            : out std_logic;  -- low active
		LUM2_ENA           : out std_logic;  --
		CAM3_POW_EN        : out std_logic;  --
		CAM3_ENA           : out std_logic;  --
		CAM3_SS            : out std_logic;  -- low active
		LUM3_ENA           : out std_logic
      );
   end component mDAB_ctrl_top ;
 

  component mDAB_mcu_signals_top is 
   port(    
      clk                : in  std_logic;  --
		mcu_data           : in  std_logic_vector (5 downto 0); -- data to be sent
		spi_run            : in  std_logic;  --
		spi_done           : out std_logic;  -- SPI cycle done
      MCU_CAL_SCLK       : out std_logic;  -- connected to MCU port  PE2, SPI clock
      MCU_CAL_MOSI       : out std_logic   -- connected to MCU port  PE6, SPI data
     );
  end component mDAB_mcu_signals_top ;
 
     signal clk               : std_logic:= '0';  --
	  signal mcu_data          : std_logic_vector (5 downto 0) := B"000000"; -- data to be sent
	  signal spi_run           : std_logic := '0';  --
	  signal spi_done          : std_logic;  -- SPI cycle done
 

     signal MCU_CAL_MISO      : std_logic ;  
     signal MCU_CAL_MOSI      : std_logic ;  
     signal MCU_CAL_SS        : std_logic := '1';   
     signal MCU_AFE_SEL       : std_logic ;   
     signal MCU_CAL_SCLK      : std_logic ;   
     signal MCU_CAL_A         : std_logic_vector(3 downto 0);  
     signal ADC_CSn           : std_logic ;  
     signal ADC_CONVSTnR      : std_logic ;
     signal ADC_SDIR          : std_logic ;
     signal ADC_SCLKR         : std_logic ;  
     signal ADC_SDO		      : std_logic ;  
     signal INTLK_2           : std_logic ;   
     signal DAC_SCLK          : std_logic ;   
     signal DAC_SYNCn         : std_logic ;   
     signal DAC_DIN           : std_logic ;  
     signal FLASH1_POW_EN     : std_logic ;  
     signal FLASH1_SELECT_CLR : std_logic ;
     signal FLASH1_SELECT_DIN : std_logic ;
     signal FLASH1_SELECT_CLK : std_logic ;  
     signal FLASH2_POW_EN     : std_logic ;  
     signal FLASH2_SELECT_CLK : std_logic ;   
     signal FLASH2_SELECT_DIN : std_logic ;   
     signal FLASH2_SELECT_CLR : std_logic ;   
     signal ILUM_POW_EN       : std_logic ;  
     signal CAM_MISO          : std_logic ;  
     signal CAM_MOSI          : std_logic ;
     signal CAM_SCLK          : std_logic ;
     signal CAM1_POW_EN       : std_logic ;  
     signal CAM1_ENA          : std_logic ;  
     signal CAM1_SS           : std_logic ;   
     signal LUM1_ENA          : std_logic ;   
     signal CAM2_POW_EN       : std_logic ;   
     signal CAM2_ENA          : std_logic ;  
     signal CAM2_SS           : std_logic ;  
     signal LUM2_ENA          : std_logic ;
     signal CAM3_POW_EN       : std_logic ;
     signal CAM3_ENA          : std_logic ;  
     signal CAM3_SS           : std_logic ;
     signal LUM3_ENA          : std_logic ;


     --signal reset              : std_logic    := 'U';
     constant clk_period       : time := 1us;
  
begin



  clk <= not clk after clk_period / 2;
 
-- DISCR_OUT_gen: process
--   variable i   : integer;
--  begin
--    while (true) loop
--      for i in 0 to 7 loop
--       DISCR_OUT_P(i) <= '1';
--       DISCR_OUT_N(i) <= '0';
--      end loop;
--       wait for 2 ns;
--      for i in 0 to 7 loop
--       DISCR_OUT_P(i) <= '0';
--       DISCR_OUT_N(i) <= '1';
--      end loop;
--      wait for 8 ns;
--    end loop;
--  end process DISCR_OUT_gen; 

 CAM_CYC_gen: process
   --variable i   : integer;
  begin
       MCU_AFE_SEL        <= '1'; -- initialization
		 INTLK_2            <= '0';
		 wait for clk_period * 10;
		 MCU_AFE_SEL        <= '0';
       wait for clk_period * 10;
		 INTLK_2            <= '1';
   -- while (true) loop
       MCU_CAL_A  <= X"F";      -- generate CAL_CLEAR_SS pulse
       MCU_CAL_SS <= '0';
       wait for clk_period * 1;
       MCU_CAL_SS <= '1';
       wait for clk_period * 10;
		 
       MCU_CAL_A  <= X"5";      -- CAL_CAM_ENA_SS = on
       MCU_CAL_SS <= '0';
       wait for clk_period * 1;
		 mcu_data   <= B"000001"; -- CAM1_ENA <='1'
		 wait for clk_period * 1;
		 spi_run    <= '1';
		 wait until spi_done = '1';
		 spi_run    <= '0';
       MCU_CAL_SS <= '1';
       wait for clk_period * 10;		 
       MCU_CAL_A  <= X"E";      -- generate CAL_LATCH_ENA pulse
       MCU_CAL_SS <= '0';
       wait for clk_period * 1;
       MCU_CAL_SS <= '1';
       wait for clk_period * 10;
		 
       MCU_CAL_A  <= X"D";      -- CAL_POWER_ENA_SS = on
       MCU_CAL_SS <= '0';
       wait for clk_period * 1;		 
		 mcu_data   <= B"000101"; -- CAM_POWER_ENA <='1'; FLASH2_POW_ENA <='1';
		 wait for clk_period * 1;
		 spi_run    <= '1';
		 wait until spi_done = '1';
		 spi_run    <= '0';
       MCU_CAL_SS <= '1';		 
       MCU_CAL_A  <= X"E";      -- generate CAL_LATCH_ENA pulse
       MCU_CAL_SS <= '0';
       wait for clk_period * 1;
       MCU_CAL_SS <= '1';
       wait for clk_period * 10;

       MCU_CAL_A  <= X"D";      -- CAL_POWER_ENA_SS = on
       MCU_CAL_SS <= '0';
       wait for clk_period * 1;		 
		 mcu_data   <= B"000010"; -- CAM_POWER_ENA <='0'; FLASH1_POW_ENA <='1'; FLASH2_POW_ENA <='0';
		 wait for clk_period * 1;
		 spi_run    <= '1';
		 wait until spi_done = '1';
		 spi_run    <= '0';
       MCU_CAL_SS <= '1';		 
       MCU_CAL_A  <= X"E";      -- generate CAL_LATCH_ENA pulse
       MCU_CAL_SS <= '0';
       wait for clk_period * 1;
       MCU_CAL_SS <= '1';
       wait for clk_period * 10;	
		 INTLK_2            <= '0'; -- check global shut down
		 
		 wait;
     -- end loop;	
	end process CAM_CYC_gen;  

 -- camera-mcu_data format:
 -- bit_0,2,4 CAM1..3_ENA
 -- bit_1,3,5 LUM1..3_ENA
 
 -- power_enable-mcu_data format:
 -- bit_0 CAM1..3_POW_ENA
 -- bit_1 FLASH1..2_POW_ENA
 
 
 mDAB_mcu_signals_top_port_map: component mDAB_mcu_signals_top port map
    (    
      clk                => clk         , 
		mcu_data           => mcu_data    ,
		spi_run            => spi_run     , 
		spi_done           => spi_done    , 
      MCU_CAL_SCLK       => MCU_CAL_SCLK, 
      MCU_CAL_MOSI       => MCU_CAL_MOSI  
     );   
 
  mDAB_ctrl_top_PORT_MAP: component mDAB_ctrl_top port map 
    (    
     MCU_CAL_MISO       => MCU_CAL_MISO,     
     MCU_CAL_MOSI       => MCU_CAL_MOSI,     
     MCU_CAL_SS         => MCU_CAL_SS,       
     MCU_AFE_SEL        => MCU_AFE_SEL,      
     MCU_CAL_SCLK       => MCU_CAL_SCLK,     
     MCU_CAL_A          => MCU_CAL_A,        
     ADC_CSn            => ADC_CSn,               
     ADC_CONVSTnR       => ADC_CONVSTnR,     
	  ADC_SDIR           => ADC_SDIR,         
	  ADC_SCLKR          => ADC_SCLKR,        
     ADC_SDO		      => ADC_SDO,		     
     INTLK_2            => INTLK_2,          
     DAC_SCLK           => DAC_SCLK,         
     DAC_SYNCn          => DAC_SYNCn,        
     DAC_DIN            => DAC_DIN,          
     FLASH1_POW_EN      => FLASH1_POW_EN,    
     FLASH1_SELECT_CLR  => FLASH1_SELECT_CLR,     
     FLASH1_SELECT_DIN  => FLASH1_SELECT_DIN,
	  FLASH1_SELECT_CLK  => FLASH1_SELECT_CLK,
	  FLASH2_POW_EN      => FLASH2_POW_EN,     
     FLASH2_SELECT_CLK  => FLASH2_SELECT_CLK,
     FLASH2_SELECT_DIN  => FLASH2_SELECT_DIN,
     FLASH2_SELECT_CLR  => FLASH2_SELECT_CLR,
     ILUM_POW_EN        => ILUM_POW_EN,      
     CAM_MISO           => CAM_MISO,         
     CAM_MOSI           => CAM_MOSI,              
     CAM_SCLK           => CAM_SCLK,         
	  CAM1_POW_EN        => CAM1_POW_EN,      
	  CAM1_ENA           => CAM1_ENA,          
     CAM1_SS            => CAM1_SS,          
     LUM1_ENA           => LUM1_ENA,         
     CAM2_POW_EN        => CAM2_POW_EN,      
     CAM2_ENA           => CAM2_ENA,         
     CAM2_SS            => CAM2_SS,          
     LUM2_ENA           => LUM2_ENA,              
     CAM3_POW_EN        => CAM3_POW_EN,      
	  CAM3_ENA           => CAM3_ENA,         
	  CAM3_SS            => CAM3_SS,           
     LUM3_ENA           => LUM3_ENA
     );
 
end architecture behavior;
