-------------------------------------------------------
-- Design Name      : mDAB_mcu_signals_top 
-- File Name        : mDAB_mcu_signals_top.vhd
-- Function         : used for simulation, generates MCU like signals
--                    
-- Coder            : K.-H. Sulanke, DESY, 2020-12-18
-------------------------------------------------------
-- Rev. 1


library IEEE;
   use IEEE.std_logic_1164.all;
   use ieee.numeric_bit.all;
   use ieee.std_logic_arith.all;
library unisim ;
   use unisim.vcomponents.all ;

entity mDAB_mcu_signals_top is 
port(    
      clk                : in  std_logic;  --
		mcu_data           : in  std_logic_vector (5 downto 0); -- data to be sent
		spi_run            : in  std_logic;  --
		spi_done           : out std_logic;  -- SPI cycle done
      MCU_CAL_SCLK       : out std_logic;  -- connected to MCU port  PE2, SPI clock
      MCU_CAL_MOSI       : out std_logic   -- connected to MCU port  PE6, SPI data
     );
end mDAB_mcu_signals_top ;

architecture arch_mDAB_mcu_signals_top of mDAB_mcu_signals_top is
 
 
   signal mcu_data_reg       : std_logic_vector (5 downto 0); -- shift register
   signal ct                 : integer range 0 to 63 := 0; 
 	 
 begin 
 
 -- SPI Bus usage with MSB first:
 
 GEN_MCU_OUTPUT: process(clk, spi_run)
   begin
	 if spi_run = '0' then
		MCU_CAL_SCLK  <= '0';
	   ct           <= 0;
		spi_done     <= '0';
		mcu_data_reg <= (others =>'0');
	 else
	  if rising_edge (clk) then	
	   if ct < 25 then
		 ct <= ct + 1;
	    if ct = 1 then
		  mcu_data_reg       <= mcu_data;
		  spi_done           <= '0';
		 end if;
		 if    (ct mod 4 = 3) then
		   MCU_CAL_SCLK <= '1';
		 elsif (ct mod 4 = 1) then
		   MCU_CAL_SCLK <= '0';
		   if (ct /= 1) then
			 mcu_data_reg(5 downto 1) <= mcu_data_reg(4 downto 0);
			end if;
       end if; --(ct mod 4 = 3)
     else
		 ct                 <= 0;
		 spi_done           <= '1';
	  end if; --if ct < 121
	 end if; --rising_edge (clk) 
	end if; --spi_run = '0'
  end process GEN_MCU_OUTPUT;
 
  MCU_CAL_MOSI <= mcu_data_reg(5);
 
   end arch_mDAB_mcu_signals_top ;
